from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth.provider import OAuthProvider
import hashlib


class HatenaAccount(ProviderAccount):
    def get_name(self):
        return self.account.extra_data.get('url_name')

    def get_avatar_url(self):
        return self.account.extra_data.get('profile_image_url')

    def to_str(self):
        name = self.get_name()
        return name or super(HatenaAccount, self).to_str()


class HatenaProvider(OAuthProvider):
    id = 'hatena'
    name = 'Hatena'
    account_class = HatenaAccount

    def get_auth_url(self, request, action):
        return 'https://www.hatena.ne.jp/oauth/authorize'

    def extract_uid(self, data):
        return data['url_name']

    def random_email(self, salt):
        return hashlib.sha224(salt.encode()).hexdigest()

    def extract_common_fields(self, data):
        return dict(id=data.get('url_name'),
                    name=data.get('display_name'),
                    email=self.random_email(data.get('url_name')),)


provider_classes = [HatenaProvider]



from setuptools import setup


requires = ["django-allauth>=0.30.0"]


setup(
    name='django-allauth-hatena',
    version='0.1.2',
    description='Allauth provider for hatena OAuth',
    url='https://github.com/h3poteto/django-allauth-hatena',
    author='h3poteto',
    author_email='h3.poteto@gmail.com',
    license='MIT',
    keywords='allauth hatena',
    packages=[
        "allauth_hatena",
    ],
    install_requires=requires,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)



import json

from allauth.socialaccount.providers.oauth.client import OAuth
from allauth.socialaccount.providers.oauth.views import (
    OAuthAdapter,
    OAuthCallbackView,
    OAuthLoginView,
)

from .provider import HatenaProvider


class HatenaAPI(OAuth):
    """
    Verifying hatena credentials
    """
    url = 'http://n.hatena.com/applications/my.json'

    def get_user_info(self):
        user = json.loads(self.query(self.url))
        return user


class HatenaOAuthAdapter(OAuthAdapter):
    provider_id = HatenaProvider.id
    request_token_url = 'https://www.hatena.com/oauth/initiate'
    access_token_url = 'https://www.hatena.com/oauth/token'
    authorize_url = 'https://www.hatena.ne.jp/oauth/authorize'

    def complete_login(self, request, app, token, response):
        client = HatenaAPI(request, app.client_id, app.secret,
                           self.request_token_url)
        # access_token for example:
        # {'oauth_token': 'hoge', 'oauth_token_secret': 'fuga', 'url_name': 'h3poteto', 'display_name': '平仮名3文字ぽてと'}

        extra_data = client.get_user_info()
        return self.get_provider().sociallogin_from_response(request,
                                                             extra_data)


oauth_login = OAuthLoginView.adapter_view(HatenaOAuthAdapter)
oauth_callback = OAuthCallbackView.adapter_view(HatenaOAuthAdapter)
